@@ -249,6 +249,15 @@ def generate_rosinstall(distro_name, names,
         logger.debug("The following wet packages found in '--from-path' will be considered: %s" % ', '.join(sorted(include_names_from_path)))
         names.update(include_names_from_path)
 
+    # Allow special keywords in repos
+    repo_names, keywords = _split_special_keywords(repo_names or [])
+    if set(keywords).difference(set([ARG_ALL_PACKAGES])):
+        raise RuntimeError('The only keyword supported by repos is %r' % (ARG_ALL_PACKAGES))
+
+    if ARG_ALL_PACKAGES in keywords:
+        wet_distro = get_wet_distro(distro_name)
+        repo_names = wet_distro.repositories.keys()
+
     # expand repository names into package names
     repo_names, unknown_repo_names = _classify_repo_names(distro_name, repo_names)
     if unknown_repo_names:
@@ -263,14 +272,14 @@ def generate_rosinstall(distro_name, names,
 
     names, unknown_names = _classify_names(distro_name, names, source=upstream_source_version)
     if unknown_names:
-        logger.warn('The following not released packages/stacks will be ignored: %s' % (', '.join(sorted(unknown_names))))
+        logger.warn('The following unreleased packages/stacks will be ignored: %s' % (', '.join(sorted(unknown_names))))
     if keywords:
         expanded_names, unknown_names = _classify_names(distro_name, _expand_keywords(distro_name, keywords), source=upstream_source_version)
         if unknown_names:
-            logger.warn('The following not released packages/stacks from the %s will be ignored: %s' % (ROS_PACKAGE_PATH, ', '.join(sorted(unknown_names))))
+            logger.warn('The following unreleased packages/stacks from the %s will be ignored: %s' % (ROS_PACKAGE_PATH, ', '.join(sorted(unknown_names))))
         names.update(expanded_names)
     if not names.wet_package_names and not names.dry_stack_names and not has_repos:
-        raise RuntimeError('No packages/stacks left after ignoring not released')
+        raise RuntimeError('No packages/stacks left after ignoring unreleased')
     if names.wet_package_names or names.dry_stack_names:
         logger.debug('Packages/stacks: %s' % ', '.join(sorted(names.wet_package_names | names.dry_stack_names)))
     if unreleased_repo_names:
@@ -280,11 +289,11 @@ def generate_rosinstall(distro_name, names,
     deps_up_to_names, keywords = _split_special_keywords(deps_up_to or [])
     deps_up_to_names, unknown_names = _classify_names(distro_name, deps_up_to_names, source=upstream_source_version)
     if unknown_names:
-        logger.warn("The following not released '--deps-up-to' packages/stacks will be ignored: %s" % (', '.join(sorted(unknown_names))))
+        logger.warn("The following unreleased '--deps-up-to' packages/stacks will be ignored: %s" % (', '.join(sorted(unknown_names))))
     if keywords:
         expanded_names, unknown_names = _classify_names(distro_name, _expand_keywords(distro_name, keywords), source=upstream_source_version)
         if unknown_names:
-            logger.warn("The following not released '--deps-up-to' packages/stacks from the %s will be ignored: %s" % (ROS_PACKAGE_PATH, ', '.join(sorted(unknown_names))))
+            logger.warn("The following unreleased '--deps-up-to' packages/stacks from the %s will be ignored: %s" % (ROS_PACKAGE_PATH, ', '.join(sorted(unknown_names))))
         deps_up_to_names.update(expanded_names)
     if deps_up_to:
         logger.debug('Dependencies up to: %s' % ', '.join(sorted(deps_up_to_names.wet_package_names | deps_up_to_names.dry_stack_names)))
@@ -298,7 +307,7 @@ def generate_rosinstall(distro_name, names,
         exclude_names.update(exclude_names_from_path)
     exclude_names, unknown_names = _classify_names(distro_name, exclude_names, source=upstream_source_version)
     if unknown_names:
-        logger.warn("The following not released '--exclude' packages/stacks will be ignored: %s" % (', '.join(sorted(unknown_names))))
+        logger.warn("The following unreleased '--exclude' packages/stacks will be ignored: %s" % (', '.join(sorted(unknown_names))))
     if keywords:
         expanded_names, unknown_names = _classify_names(distro_name, _expand_keywords(distro_name, keywords), source=upstream_source_version)
         exclude_names.update(expanded_names)
@@ -419,6 +428,9 @@ def generate_rosinstall(distro_name, names,
                         release_repo = repo.release_repository
                         assert not upstream_version_tag or release_repo.version is not None, "Package '%s' in repository '%s' does not have a release version" % (pkg_name, pkg.repository_name)
                         repos[pkg.repository_name] = repo
+            # If asked to get upstream development then the release state doesn't matter
+            if upstream_source_version:
+                repo_names = repo_names.union(unreleased_repo_names)
             for repo_name in repo_names:
                 if repo_name not in repos:
                     repos[repo_name] = wet_distro.repositories[repo_name]
